// SilkSQLite.cpp : Defines the exported functions for the DLL application.
//

#include "sqlite3.h"
#include "silklib.h"
#include<stdio.h>
#include <vector>
using namespace std;


typedef void(*dll_callback)(PARAMS* pParam);//the c++ callback in Silk Interpreter

static int callback(void *data, int argc, char **argv, char **azColName)
{
	void* pSilkCallback = NULL;
	dll_callback DllCallback = NULL;

	PARAMS* pParam = (PARAMS*)data;
	if (pParam)
	{
		if (pParam->nCount == 4)
		{
			if (pParam->pParams[2].nType == PARAM_POINTER)// the callback in your silk code
				pSilkCallback = (void*)(*(unsigned _INT*)(pParam->pParams[2].pValue));
			if (pParam->pParams[3].nType == PARAM_POINTER)//last one is the c++ callback in Silk Interpreter which will call your silk code callback
				DllCallback = (dll_callback)(*(unsigned _INT*)(pParam->pParams[3].pValue));
		}
	}

	if (DllCallback)
	{
		PARAMS* pNewParam = create_params(2);

		add_ptr(pNewParam, pSilkCallback);
		string str;
		for (int i = 0; i<argc; i++){
			//printf("%s = %s\n", azColName[i], argv[i] ? argv[i] : "NULL");
			str = str + azColName[i] + "=";
			if (argv[i])
				str += argv[i];
			else
				str += "NULL";
			str += ", ";
		}
		add_string(pNewParam, str.c_str(), str.size());//silk callback argv
		DllCallback(pNewParam);

		delete_params(pNewParam);

	}
	return 0;
}


extern "C" {

	EXPORT PARAMS* sqlite_open(PARAMS* pParam) {

		char filename[256];
		filename[0] = 0;
		if (pParam)
		{
			if (pParam->nCount == 1)
			{
				if (pParam->pParams[0].nType == PARAM_STR)
					sprintf_s(filename, "%s", (char*)(pParam->pParams[0].pValue));
			}
		}
		sqlite3* pDB;
		int nRes = sqlite3_open(filename, &pDB);
		if (nRes == SQLITE_OK)
		{
			PARAMS* pRetParam = create_params(1);
			add_ptr(pRetParam, pDB);
			return pRetParam;
		}
		return NULL;
	}
	EXPORT PARAMS* sqlite_close(PARAMS* pParam) {
		if (pParam && pParam->nCount == 1)
		if (pParam->pParams[0].nType == PARAM_POINTER)
		{
			sqlite3* pDB = (sqlite3*)(*(unsigned _INT*)(pParam->pParams[0].pValue));
			if (pDB)
				sqlite3_close(pDB);
		}
		return NULL;
	}
	EXPORT PARAMS* sqlite_exec(PARAMS* pParam) {
		sqlite3* pDB = NULL;
		char sql[256];
		sql[0] = 0;
		if (pParam)
		{
			if (pParam->nCount == 2)
			{
				if (pParam->pParams[0].nType == PARAM_POINTER)
					pDB = (sqlite3*)(*(unsigned _INT*)(pParam->pParams[0].pValue));
				if (pParam->pParams[1].nType == PARAM_STR)
					sprintf_s(sql, "%s", (char*)(pParam->pParams[1].pValue));
			}
		}
		if (pDB)
		{
			char* cErrMsg;
			int nRes = sqlite3_exec(pDB, sql, 0, 0, &cErrMsg);
			if (nRes == SQLITE_OK)
			{
				PARAMS* pRetParam = create_params(1);
				add_int(pRetParam, 1);
				return pRetParam;
			}

		}
		return NULL;
	}
	EXPORT PARAMS* sqlite_query_cb(PARAMS* pParam) {
		sqlite3* pDB = NULL;
		char sql[256];
		sql[0] = 0;
		if (pParam)
		{
			if (pParam->nCount >= 2)
			{
				if (pParam->pParams[0].nType == PARAM_POINTER)
					pDB = (sqlite3*)(*(unsigned _INT*)(pParam->pParams[0].pValue));
				if (pParam->pParams[1].nType == PARAM_STR)
					sprintf_s(sql, "%s", (char*)(pParam->pParams[1].pValue));
			}
		}
		if (pDB)
		{
			char* cErrMsg;
			int nRes = sqlite3_exec(pDB, sql, callback, pParam, &cErrMsg);
			if (nRes == SQLITE_OK)
			{
				PARAMS* pRetParam = create_params(1);
				add_int(pRetParam, 1);
				return pRetParam;
			}

		}
		return NULL;
	}
	EXPORT PARAMS* sqlite_query(PARAMS* pParam) {
		sqlite3* pDB = NULL;
		char sql[256];
		sql[0] = 0;
		if (pParam)
		{
			if (pParam->nCount == 2)
			{
				if (pParam->pParams[0].nType == PARAM_POINTER)
					pDB = (sqlite3*)(*(unsigned _INT*)(pParam->pParams[0].pValue));
				if (pParam->pParams[1].nType == PARAM_STR)
					sprintf_s(sql, "%s", (char*)(pParam->pParams[1].pValue));
			}
		}
		if (pDB)
		{
			char *errmsg = NULL;
			char ret = 0;
			char **dbResult;
			int nRow = 0, nColumn = 0;
			ret = sqlite3_get_table(pDB, sql, &dbResult, &nRow, &nColumn, &errmsg);
			if (ret == SQLITE_OK)
			{
				vector<vector<string>>* pResult = new vector < vector<string> >;
				vector<string> vecRecord;
				//save fieldname
				for (int j = 0; j < nColumn; j++)
					vecRecord.push_back(dbResult[j]);
				pResult->push_back(vecRecord);
				//save records
				int index = nColumn;
				for (int i = 0; i < nRow; i++)
				{
					vecRecord.clear();
					for (int j = 0; j < nColumn; j++)
					{
						if (dbResult[index])
							vecRecord.push_back(dbResult[index]);
						else
							vecRecord.push_back("");
						index++;
					}
					pResult->push_back(vecRecord);
				}
				sqlite3_free_table(dbResult);

				PARAMS* pRetParam = create_params(3);
				add_ptr(pRetParam, pResult);
				add_int(pRetParam, nRow);
				add_int(pRetParam, nColumn);
				return pRetParam;
			}

		}
		return NULL;
	}

	EXPORT PARAMS* sqlite_query_record(PARAMS* pParam) {
		vector<vector<string>>* pResult = NULL;
		int nRecIndex = -1;
		if (pParam)
		{
			if (pParam->nCount == 2)
			{
				if (pParam->pParams[0].nType == PARAM_POINTER)
					pResult = (vector<vector<string>>*)(*(unsigned _INT*)(pParam->pParams[0].pValue));
				if (pParam->pParams[1].nType == PARAM_INT)
					nRecIndex = *(int*)(pParam->pParams[1].pValue);
			}
		}
		if (pResult)
		{
			if (nRecIndex >= 0 && nRecIndex < pResult->size())
			{
				int nCols = (*pResult)[nRecIndex].size();
				PARAMS* pRetParam = create_params(nCols);

				for (int i = 0; i < nCols; i++)
				{
					add_string(pRetParam, (*pResult)[nRecIndex][i].c_str(), (*pResult)[nRecIndex][i].size());
				}

				return pRetParam;
			}
		}
		return NULL;
	}

	EXPORT PARAMS* sqlite_query_free(PARAMS* pParam) {
		vector<vector<string>>* pResult = NULL;
		if (pParam)
		{
			if (pParam->nCount == 1)
			{
				if (pParam->pParams[0].nType == PARAM_POINTER)
					pResult = (vector<vector<string>>*)(*(unsigned _INT*)(pParam->pParams[0].pValue));
			}
		}
		if (pResult)
			delete pResult;
		return NULL;
	}
}

